###############################################################################
# zapplet - A tray applet for interfacing with Zenoss
# Copyright (C) 2008 Nathaniel McCallum <nathaniel@natemccallum.com>
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
###############################################################################

import gtk
import gobject


class EventView(gtk.TreeView):

	def __init__(self):
		# Setup the event view
		super(EventView, self).__init__(gtk.ListStore(gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf, str, str))
		self.set_headers_visible(True)
		self.set_enable_search(True)
		self.set_search_column(1)
		self.set_search_equal_func(lambda *a: True) # TODO: Make search work
		self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

		# Setup the status column
		col = gtk.TreeViewColumn("Status", gtk.CellRendererPixbuf(), pixbuf=1)
		self.append_column(col)

		# Setup the details column
		col = gtk.TreeViewColumn("Details", gtk.CellRendererText(), markup=2)
		col.set_property("expand", True)
		self.append_column(col)

		# Setup the time column
		col = gtk.TreeViewColumn("Last Time / First Time", gtk.CellRendererText(), markup=3)
		self.append_column(col)

		# Setup the right-click menu
		def button_press(zev, event, self):
			"Popup a context menu on right click, otherwise behave normally."

			# Find the path at the point we clicked
			path = self.get_path_at_pos(int(event.x), int(event.y))

			# If this was a right-click and we clicked on an item in the view...
			if event.button == 3 and path:
				# Get the event object we clicked on
				i = self.get_model().get_iter(path[0])
				e = self.get_model().get_value(i, 0)

				# If we clicked on an event that isn't already selected,
				# move the selection to the new event
				if not e in self.get_selected_events():
					self.do_button_press_event(zev, event)

				# Build the "View details" menu item
				def view_clicked(mi, self):
					[e.view() for e in self.get_selected_events()]
				view = gtk.ImageMenuItem(gtk.STOCK_INFO)
				view.get_child().set_text("View details")
				view.connect("activate", view_clicked, self)

				# Build the "Acknowledge" menu item
				def ack_clicked(mi, self):
					m = self.get_model()
					paths = self.get_selection().get_selected_rows()[1]

					# 1. Iterate over all the selected rows
					for trr in [gtk.TreeRowReference(m, p) for p in paths]:
						# Regenerate the icon
						i = m.get_iter(trr.get_path())
						e = m.get(i, 0)[0]
						e.eventState = 1
						m.set(i, 1, e.get_pixbuf(24, 24))

						# Notify the server
						e.acknowledge()
				def ack_show(mi, self):
					events = self.get_selected_events()

					# If there are any un-acknowledged selected events, show the ack option
					if filter(lambda x: not x, [e.eventState for e in events]):
						mi.set_sensitive(True)

					# Otherwise, grey it out because we can't use it
					else:
						mi.set_sensitive(False)
				ack = gtk.ImageMenuItem(gtk.STOCK_APPLY)
				ack.get_child().set_text("Acknowledge")
				ack.connect("activate", ack_clicked, self)
				ack.connect("show", ack_show, self)

				# Build the "Move to history" menu item
				def hist_clicked(mi, self):
					m = self.get_model()
					paths = self.get_selection().get_selected_rows()[1]

					# Iterate over all the selected rows
					for trr in [gtk.TreeRowReference(m, p) for p in paths]:
						# Remove the event row
						i = m.get_iter(trr.get_path())
						e = m.get(i, 0)[0]
						m.remove(i)

						# Notify the server
						e.historify()
				hist = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
				hist.get_child().set_text("Move to history")
				hist.connect("activate", hist_clicked, self)

				# Build the menu
				menu = gtk.Menu()
				menu.append(view)
				menu.append(ack)
				menu.append(hist)
				menu.show_all()

				# Popup the menu
				menu.popup(None, None, None, event.button, event.time)
				return True

			# Behave normally, this isn't a right click
			return self.do_button_press_event(zev, event)
		self.connect("button_press_event", button_press, self)

	def set_events(self, events):
		import xml.sax.saxutils
		model = self.get_model()
		model.clear()

		for e in events:
			summary = xml.sax.saxutils.escape(e.summary.replace("\n", ""))
			text  = "<b>"
			text += e.device
			text += 0 < len(e.component) < 20 and " (%s) - " % e.component or " - "
			text += e.eventClass + "</b>\n"
			text += "<span size='small'>%s</span>" % summary
			time  = "<b>" + e.lastTime.split(".")[0] + "</b>\n" + e.firstTime.split(".")[0]
			model.append((e, e.get_pixbuf(24, 24), text, time))

	def get_selected_events(self):
		model, paths = self.get_selection().get_selected_rows()
		events = [model.get(model.get_iter(p), 0)[0] for p in paths]
		return events
